'''
Created on Apr 10, 2013

@author: Steven
'''

from django import template

register = template.Library()

def current_version():
    from hmf.Perturbations import version
    
    return version

register.simple_tag(current_version)


'''
Created on May 3, 2012

@author: smurray
'''

from django import forms
from django.utils.safestring import mark_safe
import numpy as np

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Fieldset, ButtonHolder, Submit, Div
from crispy_forms.bootstrap  import TabHolder, Tab, InlineCheckboxes, FormActions
#--------- Custom Form Field for Comma-Separated Input -----
class FloatListField(forms.CharField):
    """
    Defines a form field that accepts comma-separated real values and returns a list of floats.
    """
    def __init__(self, min_val=None, max_val=None, *args, **kwargs):
        self.min_val, self.max_val = min_val, max_val
        super(FloatListField, self).__init__(*args, **kwargs)

    def clean(self, value):
        value = self.to_python(value)
        self.validate(value)
        self.run_validators(value)

        final_list = []
        if value:
            numbers = value.split(',')
            for number in numbers:
                try:
                    final_list.append(float(number))
                except ValueError:
                    raise forms.ValidationError("%s is not a float" % number)

            for number in final_list:
                if self.min_val is not None:
                    if number < self.min_val:
                        raise forms.ValidationError("z must be greater than " + str(self.min_val) + " (" + str(number) + ")")
                if self.max_val is not None:
                    if number > self.max_val:
                        raise forms.ValidationError("z must be smaller than " + str(self.max_val) + " (" + str(number) + ")")

        return final_list

#------------ THE BIG FORM ------------------------------#
class HMFInput(forms.Form):
    """
    Input parameters to the halo mass function finder.
    """
    #------ Init Method for Dynamic Form -------------
    def __init__(self, add, minm=None, maxm=None, *args, **kwargs):
        self.add = add
        self.minm = minm
        self.maxm = maxm
        super (HMFInput, self).__init__(*args, **kwargs)

        if add == 'create':
            # Then we wnat to display min_M and max_M
                # Which values of the radius to use?
            self.fields['min_M'] = forms.FloatField(label="Minimum Mass",
                                                    initial=8.0,
                                                    help_text=mark_safe("Units of log<sub>10</sub>(M<sub>&#9737</sub>)"),
                                                    min_value=3.0,
                                                    max_value=18.0)
            self.fields['max_M'] = forms.FloatField(label="Maximum Mass",
                                                    initial=15.0,
                                                    help_text=mark_safe("Units of log<sub>10</sub>(M<sub>&#9737</sub>)"),
                                                    min_value=3.0,
                                                    max_value=18.0)
            self.fields['M_step'] = forms.FloatField(label="Mass Bin Width",
                                                     initial=0.05,
                                                     help_text="Logarithmic Bins",
                                                     min_value=0.00001,
                                                     max_value=15.0)

        self.helper = FormHelper()
        self.helper.form_id = 'input_form'
        self.helper.form_class = 'form-horizontal'
        self.helper.form_method = 'post'
        self.helper.form_action = ''
        self.helper.help_text_inline = True

        if add == 'create':
            # self.helper.add_input(Submit('submit', 'Calculate!'))
            self.helper.layout = Layout(
                                        TabHolder(
                                                  Tab('Run Parameters',
                                                      Div(
                                                          Div('z',
                                                              'overdensity',
                                                              'WDM',
                                                              'approach',
                                                              'alternate_model',
                                                              css_class='span4'
                                                              ),
                                                           Div('extrapolate',
                                                               'k_begins_at',
                                                               'k_ends_at',
                                                               'min_M',
                                                               'max_M',
                                                               'M_step',
                                                               css_class='span4'
                                                               )),
                                                      Div(
                                                          Fieldset('Optional Extra Plots',
                                                                   InlineCheckboxes('extra_plots'),
                                                                   ),
                                                          css_class='span12'

                                                          )
                                                      ),
                                                  Tab('Cosmological Parameters',
                                                      Div(
                                                          Div('co_transfer_file',
                                                              'co_transfer_file_upload',
                                                              'cp_label',
                                                              'cp_delta_c',
                                                              'cp_n',

                                                              css_class='span4'
                                                              ),
                                                          Div('cp_sigma_8',
                                                              'cp_H0',
                                                              'cp_omegab',
                                                              'cp_omegac',
                                                              'cp_omegav',
#                                                              'cp_w_lam',
                                                              'cp_omegan',

                                                              css_class='span4'
                                                              )
                                                          )
                                                      )
                                                  ),
                                        FormActions(Submit('submit', 'Calculate!', css_class='btn btn-primary btn-large'))
                                        )
        else:
            self.helper.layout = Layout(
                                        TabHolder(
                                                  Tab('Run Parameters',
                                                      Div(
                                                          Div('z',
                                                              'overdensity',
                                                              'WDM',
                                                              'approach',
                                                              css_class='span4'
                                                              ),
                                                          Div('extrapolate',
                                                              'k_begins_at',
                                                              'k_ends_at',
                                                              'alternate_model',
                                                              css_class='span4'
                                                              )),
                                                      Div(
                                                          Fieldset('Optional Extra Plots',
                                                                   InlineCheckboxes('extra_plots'),
                                                                   ),
                                                          css_class='span12'

                                                          )
                                                      ),
                                                  Tab('Cosmological Parameters',
                                                      Div(
                                                          Div('co_transfer_file',
                                                              'co_transfer_file_upload',
                                                              'cp_label',
                                                              'cp_delta_c',
                                                              'cp_n',

                                                              css_class='span4'
                                                              ),
                                                          Div('cp_sigma_8',
                                                              'cp_H0',
                                                              'cp_omegab',
                                                              'cp_omegac',
                                                              'cp_omegav',
#                                                              'cp_w_lam',
                                                              'cp_omegan',

                                                              css_class='span4'
                                                              )
                                                          )
                                                      )
                                                  ),
                                        FormActions(Submit('submit', 'Calculate!', css_class='btn btn-primary btn-large'))
                                        )

    ###########################################################
    # MAIN RUN PARAMETERS
    ###########################################################
    # Redshift at which to calculate the mass variance.
    z = FloatListField(label="Redshifts",
                       initial='0',
                       help_text="Comma-separated list",
                       max_length=50,
                       min_val=0,
                       max_val=1100)


    overdensity = FloatListField(label="Virial Overdensity",
                                 help_text="Comma-separated list",
                                 max_length=50,
                                 initial=178,
                                 min_val=10)

    # WDM particle masses (empty list if none)
    WDM = FloatListField(label="WDM Masses",
                         required=False,
                         help_text="Comma-separated list. In keV (eg. 0.05)",
                         max_length=50,
                         min_val=0.0001)


    # Mass Function fit
    approach_choices = [("PS", "Press-Schechter (1974)"),
                        ("ST", "Sheth-Tormen (2001)"),
                        ("Jenkins", "Jenkins (2001)"),
                        ("Reed03", "Reed (2003)"),
                        ("Warren", "Warren (2006)"),
                        ("Reed07", "Reed (2007)"),
                        ("Tinker", "Tinker (2008)"),
                        ("Crocce", "Crocce (2010)"),
                        ("Courtin", "Courtin (2010)"),
                        ("Angulo", "Angulo (2012)"),
                        ("Angulo_Bound", "Angulo (Subhaloes) (2012)"),
                        ("Watson_FoF", "Watson (FoF Universal) (2012)"),
                        ("Watson", "Watson (Redshift Dependent) (2012)"),
                        ]

    approach = forms.MultipleChoiceField(label="Fitting Function",
                                         choices=approach_choices,
                                         initial=['ST'],
                                         required=False)

    alternate_model = forms.CharField(label=mark_safe('Custom Fitting Function'),
                                       help_text=mark_safe('Type a fitting function form (<a href="http://docs.scipy.org/doc/numpy/reference/routines.math.html">Python syntax</a>) in terms of mass variance (denoted by x). Eg. for Jenkins: 0.315*exp(-abs(log(1.0/x)+0.61)**3.8)'),
                                       required=False,
                                       widget=forms.Textarea(attrs={'cols':'40', 'rows':'3'}))

    transfer_choices = [('transfers/PLANCK_transfer.dat', 'PLANCK'),
                        ('transfers/WMAP9_transfer.dat', 'WMAP9'),
                        ("transfers/WMAP7_transfer.dat", "WMAP7"),
                        ("transfers/WMAP5_transfer.dat", "WMAP5"),
                        ('transfers/WMAP3_transfer.dat', 'WMAP3'),
                        ('transfers/WMAP1_transfer.dat', 'WMAP1'),
                        ("transfers/Millennium_transfer.dat", "Millennium (and WALLABY)"),
                        ("transfers/GiggleZ_transfer.dat", "GiggleZ"),
                        ("custom", "Custom")]

    co_transfer_file = forms.ChoiceField(label="Transfer Function",
                                choices=transfer_choices,
                                initial="transfers/WMAP7_transfer.dat")

    co_transfer_file_upload = forms.FileField(label="Upload Transfer Function",
                                              required=False,
                                              help_text="Custom file only used if Transfer Functions is 'Custom'")

    def clean_co_transfer_file_upload(self):
        thefile = self.cleaned_data['co_transfer_file_upload']
        if thefile is not None:
            try:
                np.genfromtxt(thefile)
            except:
                raise forms.ValidationError("Uploaded transfer file is of the wrong format")
#===================================================================
#    RUN PARAMETERS
#===================================================================
    # Extrapolation parameters.
    extrapolate = forms.BooleanField(label='Extrapolate bounds of k?', initial=True, required=False)
    k_ends_at = FloatListField(label="Maximum k",
                               initial=2000.0,
                               help_text="Highest Wavenumber Used, Comma-Separated Decimals",
                               min_val=0.1)

    k_begins_at = FloatListField(label="Minimum k",
                                 initial=0.00000001,
                                 help_text="Lowest Wavenumber Used",
                                 max_val=10,
                                 min_val=0.0)

#===================================================================
#    OPTIONAL PLOTS
#===================================================================
    optional_plots = [("get_ngtm", "N(>M)"),
                        ("get_mgtm", "M(>M)"),
                        ("get_nltm", "N(<M)"),
                        ("get_mltm", "M(<M)"),
                        ("get_L", 'Box Size for One Halo'),
                        ]

    extra_plots = forms.MultipleChoiceField(label="Optional Extra Plots",
                                            choices=optional_plots,
                                            initial=['get_ngtm'],
                                            required=False)

#===================================================================
#   COSMOLOGICAL PARAMETERS
#===================================================================
    cp_label = forms.CharField(label="Unique Labels",
                               initial='WMAP7',
                               help_text="One unique identifier for each cosmology, separated by commas")

    def clean_cp_label(self):
        labels = self.cleaned_data['cp_label']
        labels = labels.split(',')
        for i, label in enumerate(labels):
            lab = label.strip()
            lab = lab.replace(" ", "_")
            labels[i] = lab
        return labels

    # Critical Overdensity corresponding to spherical collapse
    cp_delta_c = FloatListField(label=mark_safe("&#948<sub>c</sub>"),
                                  initial='1.686',
                                  min_val=1,
                                  max_val=3)
    # Power spectral index
    cp_n = FloatListField(label=mark_safe("n<sub>s</sub> "),
                          initial='0.967',
                          min_val= -4,
                          max_val=3)

    # Mass variance on a scale of 8Mpc
    cp_sigma_8 = FloatListField(label=mark_safe("&#963<sub>8</sub>"),
                                initial='0.81',
                                min_val=0.1)

    # Hubble Constant
    cp_H0 = FloatListField(label=mark_safe("H<sub>0</sub>"),
                               initial='70.4',
                               min_val=10,
                               max_val=500.0)

    cp_omegab = FloatListField(label=mark_safe("&#937<sub>b</sub>"),
                                       initial='0.0455',
                                       min_val=0.005,
                                       max_val=0.65)

    cp_omegac = FloatListField(label=mark_safe("&#937<sub>c</sub>"),
                                       initial='0.226',
                                       min_val=0.02,
                                       max_val=2.0)

    cp_omegav = FloatListField(label=mark_safe("&#937<sub>&#923</sub>"),
                                       initial='0.728',
                                       min_val=0,
                                       max_val=1.6)

#    cp_w_lam = FloatListField(label="w",
#                                       initial='-1.0')

    cp_omegan = FloatListField(label=mark_safe("&#937<sub>v</sub>"),
                                       initial='0.0',
                                       min_val=0,
                                       max_val=0.7)

    def clean(self):
        cleaned_data = super(HMFInput, self).clean()

        #========= Check At Least One Approach Is Chosen ======#
        approach = cleaned_data.get("approach")
        alternate_model = cleaned_data.get("alternate_model")

        if not approach and not alternate_model:
            raise forms.ValidationError("You must either choose an approach or enter a custom fitting function")

        #========= Check That There are Enough Labels =========#
        labels = cleaned_data.get("cp_label")
        cosmo_quantities = []
        for key, val in cleaned_data.iteritems():
            if key.startswith('cp_'):
                cosmo_quantities.append(key)

        lengths = []
        for quantity in cosmo_quantities:
            lengths = lengths + [len(cleaned_data.get(quantity))]

        if len(labels) != max(lengths):
            raise forms.ValidationError("There must be %s labels separated by commas" % max(lengths))

        #========== Check That k limits are right ==============#
        extrapolate = cleaned_data.get("extrapolate")
        if extrapolate:
            min_k = cleaned_data.get("k_begins_at")
            max_k = cleaned_data.get("k_ends_at")
            num_k_bounds = np.max([len(min_k), len(max_k)])
            for i in range(num_k_bounds - 1):
                mink = min_k[np.min([len(min_k) - 1, i])]
                maxk = max_k[np.min([len(max_k) - 1, i])]
                if maxk < mink:
                    raise forms.ValidationError("All min(k) must be less than max(k)")

        #=========== Check that Mass limits are right ==========#
        if not self.minm:
            minm = cleaned_data.get("min_M")
            maxm = cleaned_data.get("max_M")
            mstep = cleaned_data.get("M_step")
            if maxm < minm:
                raise forms.ValidationError("min(M) must be less than max(M)")
            if mstep > maxm - minm:
                raise forms.ValidationError("Mass bin width must be less than the range of Mass")

        #=========== Here we check roughly how long we expect calculations to take and make the user adjust if too long
        #        For 50 M's:
        #----------------------------------------------------
        #setup             :  0.903198504448
        #set_transfer_cosmo:  1.05522716045
        #set_kbounds       :  0.0717063983281
        #set_WDM           :  0.071452331543
        #set_z             :  0.00227285861969
        #Get MF            :  6.63149356842e-05
        #----------------------------------------------------
        initial_setup_time = 0.9
        set_transfer_time = max((len(cleaned_data.get("cp_H0")) , len(cleaned_data.get("cp_omegab")), len(cleaned_data.get("cp_omegac")),
                                 len(cleaned_data.get("cp_omegav")), len(cleaned_data.get("cp_omegan")))) * 1.055

        set_kbounds_time = max((len(cleaned_data.get("k_ends_at")), len(cleaned_data.get("k_begins_at")))) * len(cleaned_data.get("cp_n")) * \
                            len(cleaned_data.get("cp_sigma_8")) * 0.072

        set_WDM_time = len(cleaned_data.get("WDM")) * 0.072

        set_z_time = len(cleaned_data.get("z")) * 0.003

        get_mf_time = len(cleaned_data.get("approach")) * len(cleaned_data.get("overdensity")) * len(cleaned_data.get("cp_delta_c")) * 6.64 * 10 ** -5

        total_time = initial_setup_time + set_transfer_time + set_kbounds_time + set_WDM_time + set_z_time + get_mf_time

        if total_time > 10.0:
            raise forms.ValidationError("Your choice of data will take too long to calculate, please reduce the amounts of combinations")
        return cleaned_data



class PlotChoice(forms.Form):

    def __init__(self, request, *args, **kwargs):
        super (PlotChoice, self).__init__(*args, **kwargs)
        # Add in extra plot choices if they are required by the form in the session.
        session_plots = request.session['extra_plots']
        extra_plots = []
        if 'get_ngtm' in session_plots:
            extra_plots.append(("ngtm", "N(>M)"))
        if 'get_nltm' in session_plots:
            extra_plots.append(("nltm", "N(<M)"))
        if "get_mgtm" in session_plots:
            extra_plots.append(("Mgtm", "Mass(>M)"))
        if "get_mltm" in session_plots:
            extra_plots.append(("Mltm", "Mass(<M)"))
        if "get_L" in session_plots:
            extra_plots.append(("L", "Box Size for One Halo"))

        plot_choices = [("hmf", "Mass Function"),
                        ("f", "f(sigma)"),
                        ("sigma", "Mass Variance"),
                        ("lnsigma", "ln(1/sigma)"),
                        ("n_eff", "Effective Spectral Index"),
                        ("comparison_hmf", "Comparison of Mass Functions"),
                        ("comparison_f", "Comparison of Fitting Functions"),
                        ("mhmf", "Mass by Mass Function"),
                        ("power_spec", "Power Spectrum")] + extra_plots

        self.fields["plot_choice"] = forms.ChoiceField(label="View plot of",
                                        choices=plot_choices,
                                        initial='hmf')

        self.helper = FormHelper()
        self.helper.form_id = 'plotchoiceform'
        self.helper.form_class = 'form-horizontal'
        self.helper.form_method = 'post'
        self.helper.form_action = ''
        self.helper.help_text_inline = True

        self.helper.layout = Layout(Div('plot_choice', 'download_choice', css_class="span3"))

    download_choices = [("pdf-current", "PDF of Current Plot"),
                    ("pdf-all", "PDF's of All Plots"),
                    ("ASCII-mass", "ASCII table of all functions of mass"),
                    ("ASCII-k", "ASCII table of all functions of wavenumber"),
                    ("parameters", "List of parameter values")]

    download_choice = forms.ChoiceField(label=mark_safe('<a href="../hmf.pdf" id="plot_download">Download </a>'),
                                choices=download_choices,
                                initial='pdf-current')




'''
Created on Jun 15, 2012

@author: Steven
'''
from hmf.Perturbations import Perturbations
from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
from matplotlib.figure import Figure
import numpy as np
import logging
import cosmolopy.distance as cd
import pandas
import os

def hmf_driver(transfer_file,  #File produced by CAMB containing the transfer function.
               extrapolate,  #Bool - whether to extrapolate power spectrum
               k_bounds,  #Bounds to extrpolate powe spec to.
               z_list,  #Redshifts
               WDM_list,  #WDM masses
               approaches,  #Fitting Functions
               overdensities,  #Virial overdensity parameters
               cosmology_list,  #List of cosmology dictionaries
               min_M, max_M,  #Minimum and Maximum mass to calculate for
               M_step,  #Step size in log10(M_sun)
               user_model,  #An optional mass function model from the user
               cosmo_labels,  #Labels for each of the cosmologies
               extra_plots):  #A dictionary of bools for optional extra plots.

    # Change directory to this file (for picking up transfer files if pre-made)
    os.chdir(os.path.dirname(__file__))

    #Set-up array of masses
    masses = np.arange(min_M, max_M, M_step)

    #Create a dataframe to hold the mass-based data
    mass_data = pandas.DataFrame({'M':10 ** masses})

    #Create a table to hold the k-based data
    k_data = pandas.DataFrame(index=range(4097))

    labels = {}
    warnings = {}

    pert = Perturbations(M=masses,
                         transfer_file=transfer_file,
                         z=z_list[0],
                         WDM=None,
                         k_bounds=k_bounds[0],
                         extrapolate=extrapolate,
                         reion__use_optical_depth=True,
                         **cosmology_list[0])

    #Loop through all the different cosmologies
    for cosmo_i, cosmo_dict in enumerate(cosmology_list):
        #Save the cosmo_label to the column label
        labels['cosmo'] = cosmo_labels[cosmo_i]
        #Loop through all WDM models (CDM first)
        for k_bound in k_bounds:
            #Save the k_bounds to the label
            if len(k_bounds) > 1:
                labels['k'] = 'k{' + str(k_bound[0]) + ',' + str(k_bound[1]) + '}'
            for wdm in [None] + WDM_list:
                #Add WDM label
                if len(WDM_list) > 0:
                    if wdm is None:
                        labels['wdm'] = 'CDM'
                    else:
                        labels['wdm'] = 'WDM=' + str(wdm)
                #Loop over all redshifts
                for z in z_list:
                    #Define a column-name extension for the table
                    if len(z_list) > 1 or z > 0:
                        labels['z'] = 'z=' + str(z)

                    print cosmo_dict
                    #Update pert object optimally with new variables
                    pert.update(k_bounds=k_bound, WDM=wdm, z=z, **cosmo_dict)


                    #Save k-based data
                    k_data["k_" + getname(labels, excl=['deltavir', 'fsig'])] = pert.k
                    k_data["P(k)_" + getname(labels, excl=['deltavir', 'fsig'])] = pert.power_spectrum

                    #Save Mass-Based data
                    mass_data["sigma_" + getname(labels, excl=['deltavir', 'fsig'])] = pert.sigma
                    mass_data["lnsigma_" + getname(labels, excl=['deltavir', 'fsig'])] = pert.lnsigma
                    mass_data["neff_" + getname(labels, excl=['deltavir', 'fsig'])] = pert.n_eff

                    #Loop over fitting functions
                    for approach in approaches:
                        labels['fsig'] = approach
                        for overdensity in overdensities:
                            if len(overdensities) > 1:
                                labels['deltavir'] = 'Dvir=' + str(overdensity)

                            #Save the data
                            mass_func = pert.MassFunction(fsigma=approach, overdensity=overdensity, delta_c=cosmo_dict['delta_c'])

                            mass_data["hmf_" + getname(labels)] = mass_func
                            mass_data["f(sig)_" + getname(labels)] = pert.vfv
                            mass_data["M*hmf_" + getname(labels)] = mass_func * pert.M

                            #Easily add more when you need to
                            if 'get_ngtm' in extra_plots:
                                #mass_data.add_column("NgtM_"+name_ext, pert.NgtM(mass_func),unit="h^3/Mpc^3")
                                mass_data["NgtM_" + getname(labels)] = pert.NgtM(mass_func)
                            if 'get_mgtm' in extra_plots:
                                #mass_data.add_column("MgtM_"+name_ext, pert.MgtM(mass_func),unit="log10(M_sun) h^2/Mpc^3")
                                mass_data["MgtM_" + getname(labels)] = pert.MgtM(mass_func)
                            if 'get_nltm' in extra_plots:
                                #mass_data.add_column("NltM_"+name_ext, pert.NltM(mass_func),unit="h^3/Mpc^3")
                                mass_data["NltM_" + getname(labels)] = pert.NltM(mass_func)
                            if 'get_mltm' in extra_plots:
                                #mass_data.add_column("MltM_"+name_ext, pert.MltM(mass_func),unit="log10(M_sun) h^3/Mpc^3")
                                mass_data["MltM_" + getname(labels)] = pert.MltM(mass_func)
                            if 'get_L' in extra_plots:
                                mass_data["L(N=1)_" + getname(labels)] = pert.how_big(mass_func)

            if pert.max_error:
                warnings[getname(labels, excl=['deltavir', 'fsig', 'z', 'wdm'])] = [pert.max_error]
            if pert.min_error:
                warnings[getname(labels, excl=['deltavir', 'fsig', 'z', 'wdm'])].append(pert.min_error)


    return mass_data, k_data, warnings


def cosmography(cosmology_list, cosmo_labels, redshifts):

    ######################################################
    # COSMOGRAPHY
    ######################################################
    distances = []
    for i, cosmology in enumerate(cosmology_list):
        #Set a cosmology for cosmolopy
        cosmo = {'omega_M_0':cosmology['omegab'] + cosmology['omegac'],
                 'omega_lambda_0':cosmology['omegav'],
                 'h':cosmology['H0'] / 100.0,
                 'omega_b_0':cosmology['omegab'],
                 'omega_n_0':cosmology['omegan'],
                 'N_nu':0,
                 'n':cosmology['n'],
                 'sigma_8':cosmology['sigma_8']}

        cd.set_omega_k_0(cosmo)

        #Age of the Universe
        for z in redshifts:
            distances = distances + [[cosmo_labels[i],
                                      z,
                                      cd.age(z, use_flat=False, **cosmo) / (3600 * 24 * 365.25 * 10 ** 9),
                                      cd.comoving_distance(z, **cosmo) / 1000,
                                      cd.comoving_volume(z, **cosmo) / 10 ** 9,
                                      cd.angular_diameter_distance(z, **cosmo) / 1000,
                                      cd.luminosity_distance(z, **cosmo) / 1000]]



    return distances

def create_canvas(masses, mass_data, title, xlab, ylab, yscale):
    ######################################################
    # IMAGE (CANVAS) PLOTTING
    ######################################################
    fig = Figure(figsize=(11, 6), edgecolor='white', facecolor='white', dpi=100)
    ax = fig.add_subplot(111)
    ax.set_title(title)
    ax.grid(True)
    ax.set_xlabel(xlab)
    ax.set_ylabel(ylab)


#    mass_data.plot(ax=ax, legend=False)
    linecolours = ('b', 'g', 'r', 'c', 'm', 'y', 'k')
    lines = ["-", "--", "-.", ":"]

    counter = 0
    print mass_data
    for column in mass_data.columns:
        ax.plot(masses, mass_data[column],
                color=linecolours[(counter / 4) % 7],
                linestyle=lines[counter % 4],
                label=column.split("_", 1)[1].replace("_", ", ")
                )
        print column.split("_", 1)[1].replace("_", ", ")
        counter = counter + 1
        print "Legend info... ", counter
        print ax.get_legend_handles_labels()


    # Shrink current axis by 30%
    ax.set_xscale('log')
    ax.set_yscale(yscale)
    box = ax.get_position()
    ax.set_position([box.x0, box.y0, box.width * 0.6, box.height])

    # Put a legend to the right of the current axis
    ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))
    print "After..."
    print ax.get_legend_handles_labels()[1]
    #fig.tight_layout()

    canvas = FigureCanvas(fig)
    return canvas


def create_k_canvas(k_data, k_keys, p_keys, title, xlab, ylab):
    fig = Figure(figsize=(12, 6.7), edgecolor='white', facecolor='white')
    ax = fig.add_subplot(111)

    ax.set_title(title)
    ax.grid(True)
    ax.set_xlabel(xlab)
    ax.set_ylabel(ylab)

    linecolours = ('b', 'g', 'r', 'c', 'm', 'y', 'k')
    lines = ["-", "--", "-.", ":"]

    counter = 0
    for j, k_key in enumerate(k_keys):
        ax.plot(np.exp(k_data[k_key]), np.exp(k_data[p_keys[j]]),
                    color=linecolours[counter / 4],
                    linestyle=lines[counter % 4],
                    label=k_key.split("_", 1)[1].replace("_", ", "))
        counter = counter + 1

    #Make the axes logarithmic
    ax.set_yscale('log')
    ax.set_xscale('log')
    # Shrink current axis by 20%
    box = ax.get_position()
    ax.set_position([box.x0, box.y0, box.width * 0.6, box.height])
    # Put a legend to the right of the current axis
    ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))
    canvas = FigureCanvas(fig)
    return canvas

def getname(names, excl=[]):
    """
    Compiles the individual labels from a dictionary to a string label
    """
    label = ''
    for key, val in names.iteritems():
        if key not in excl:
            label = label + val + '_'

    label = label[:-1]

    return label



from django.http import HttpResponse  #, HttpResponseRedirect
#from django.shortcuts import render  # ,get_object_or_404, render_to_response, redirect
# from django.utils.encoding import iri_to_uri
# from django.core.context_processors import csrf
# from django.template import RequestContext
import utils
import forms
from django.views.generic.edit import FormView
#from django.views.generic.base import TemplateView
#from django.core.files.base import ContentFile
#from django.core.files.storage import default_storage
#import numpy as np
import datetime
# import logging
import StringIO
import zipfile
#import time
import os
# import atpy
import pandas
from tabination.views import TabView
from hmf.Perturbations import version
from django.conf import settings

# def index(request):
#   return HttpResponseRedirect('/admin/')

class BaseTab(TabView):
    """Base class for all main navigation tabs."""
    tab_group = 'main'
    top = True

class home(BaseTab):
    """
    The home-page. Should just be simple html with links to what to do.
    """

    _is_tab = True
    tab_id = '/'
    tab_label = 'Home'
    template_name = 'home.html'


class InfoParent(BaseTab):
    _is_tab = True
    tab_id = 'info'
    tab_label = 'Info'
    template_name = 'doesnt_exist.html'
    my_children = ['/hmf_parameters/', '/hmf_resources/', '/hmf_acknowledgments/', '/hmf_parameter_discussion/', '/contact_info/']

class InfoChild(BaseTab):
    """Base class for all child navigation tabs."""
    tab_parent = InfoParent

class parameters(InfoChild):
    """
    A simple html 'end-page' which shows information about parameters used.
    """
    _is_tab = True
    tab_id = '/hmf_parameters/'
    tab_label = 'Parameter Defaults'
    template_name = 'parameters.html'
    top = False

class contact(InfoChild):
    """
    A simple html 'end-page' which shows information about parameters used.
    """
    _is_tab = True
    tab_id = '/contact_info/'
    tab_label = 'Contact Info'
    template_name = 'contact_info.html'
    top = False

class resources(InfoChild):
    """
    A simple html 'end-page' which shows information about parameters used.
    """
    _is_tab = True
    tab_id = '/hmf_resources/'
    tab_label = 'Extra Resources'
    template_name = 'resources.html'
    top = False

class acknowledgments(InfoChild):
    """
    A simple html 'end-page' which shows information about parameters used.
    """
    _is_tab = True
    tab_id = '/hmf_acknowledgments/'
    tab_label = 'Acknowledgments'
    template_name = 'acknowledgments.html'
    top = False

class param_discuss(InfoChild):
    _is_tab = True
    tab_id = '/hmf_parameter_discussion/'
    tab_label = 'Parameter Info'
    template_name = 'parameter_discuss.html'
    top = False


class HMFInputBase(FormView):
    """
    The form for input. 
    """

    #Define the needed variables for FormView class
    form_class = forms.HMFInput
    success_url = '../../hmf_image_page/'
    template_name = 'hmfform.html'

    def get_success_url(self):
        if not self.mf_error and not self.dist_error and not self.merge_error:
            return '../../hmf_image_page/'
        else:
            return '../../error_page/'

    #Define what to do if the form is valid.
    def form_valid(self, form):
        # log = logging.getLogger(__name__)

        ###############################################################
        # FORM DATA MANIPULATION
        ###############################################################
        #==================== Now save and merge the cosmology ===============================================================
        cosmo_quantities = [key for key in form.cleaned_data.keys() if key.startswith('cp_')]
        n_cosmologies = len(form.cleaned_data['cp_label'])


        cosmology_list = []
        for i in range(n_cosmologies):
            cosmology_list = cosmology_list + [{}]
            for quantity in cosmo_quantities:
                index = min(len(form.cleaned_data[quantity]) - 1, i)
                cosmology_list[i][quantity[3:]] = form.cleaned_data[quantity][index]

        #==================== We make sure the cosmologies are all unique (across adds) =================================
        if self.request.path.endswith('create/'):
            # If we are creating for the first time, just save the labels to the session
            self.request.session["cosmo_labels"] = form.cleaned_data["cp_label"]  # A compressed list of all unique labels
            self.request.session["cosmologies"] = cosmology_list  # A compressed list of dicts of unique parameters.

        elif self.request.path.endswith('add/'):
            # Go through each new label/cosmology added to check uniqueness
            for i, label in enumerate(form.cleaned_data["cp_label"]):
                counter = 0

                # If the label is not unique across adds
                if label in self.request.session['cosmo_labels']:
                    # Extract the cosmological parameters associated with this label from the previous add
                    cosmology = self.request.session['cosmologies'][self.request.session['cosmo_labels'].index(label)]

                    same = True
                    # Check if all the parameters from the old add are the same as the new add. If not, same = False.
                    for key in cosmology_list[i].keys():
                        same = same and cosmology_list[i][key] == cosmology[key]

                    # If the cosmologies aren't the same, then we modify the label to be unique by suffixing a unique integer
                    while label in self.request.session['cosmo_labels'] and not same:
                        counter = counter + 1

                        if counter == 1:
                            # Add a number to the end to make it unique
                            label = label + '(' + str(counter) + ')'
                        else:
                            label = label[:-3] + '(' + str(counter) + ')'

                    # If the cosmologies are unique, the labels are now also unique, so save the new cosmology/label to session.
                    if not same:
                        self.request.session["cosmo_labels"].append(label)
                        self.request.session['cosmologies'].append(cosmology_list[i])

                else:
                    # The label is already unique so just append it to session
                    self.request.session["cosmo_labels"].append(label)
                    self.request.session['cosmologies'].append(cosmology_list[i])

                # Change the label in the form
                form.cleaned_data["cp_label"][i] = label


            form.cleaned_data['min_M'] = self.request.session['min_M']
            form.cleaned_data['max_M'] = self.request.session['max_M']
            form.cleaned_data['M_step'] = self.request.session['M_step']

        #=========== Set the Transfer Function File correctly ===== #
        transfer_file = form.cleaned_data["co_transfer_file"]
        if transfer_file == 'custom':
            if form.cleaned_data["co_transfer_file_upload"] == None:
                transfer_file = None
            else:
                transfer_file = form.cleaned_data['co_transfer_file_upload']

        #=========== Set k-bounds as a list of tuples ==============================#
        min_k = form.cleaned_data['k_begins_at']
        max_k = form.cleaned_data['k_ends_at']
        num_k_bounds = max(len(min_k), len(max_k))
        k_bounds = []
        for i in range(num_k_bounds):
            mink = min_k[min(len(min_k) - 1, i)]
            maxk = max_k[min(len(max_k) - 1, i)]
            k_bounds.append((mink, maxk))

        #============ Set other simple data ========================================#
        approach = []
        if form.cleaned_data['approach']:
            for i in form.cleaned_data["approach"]:
                approach = approach + [str(i)]

        if form.cleaned_data["alternate_model"]:
            approach = approach + ['user_model']

        # DO THE CALCULATIONS
        mass_data, k_data, warnings = utils.hmf_driver(transfer_file=transfer_file,
                                                       extrapolate=form.cleaned_data['extrapolate'],
                                                       k_bounds=k_bounds,
                                                       z_list=form.cleaned_data['z'],
                                                       WDM_list=form.cleaned_data['WDM'],
                                                       approaches=approach,
                                                       overdensities=form.cleaned_data['overdensity'],
                                                       cosmology_list=cosmology_list,
                                                       min_M=form.cleaned_data['min_M'], max_M=form.cleaned_data['max_M'],
                                                       M_step=form.cleaned_data['M_step'],
                                                       user_model=form.cleaned_data['alternate_model'],
                                                       cosmo_labels=form.cleaned_data['cp_label'],
                                                       extra_plots=form.cleaned_data['extra_plots'])

        distances = utils.cosmography(cosmology_list, form.cleaned_data['cp_label'], form.cleaned_data['z'])

        # Delete the uploaded file. Not needed anymore. Maybe should leave it here for adds??.
        if form.cleaned_data["co_transfer_file_upload"] != None:
            os.system("rm " + form.cleaned_data["co_transfer_file_upload"])


        if self.request.path.endswith('add/'):
            self.request.session["mass_data"] = pandas.concat([self.request.session["mass_data"], mass_data], join='inner', axis=1)
            self.request.session["k_data"] = pandas.concat([self.request.session["k_data"], k_data], join='inner', axis=1)
            self.request.session['distances'] = self.request.session['distances'] + [distances]
            self.request.session['input_data'] = self.request.session['input_data'] + [form.cleaned_data]
            self.request.session['warnings'].update(warnings)
            self.request.session['extra_plots'] = list(set(form.cleaned_data['extra_plots'] + self.request.session['extra_plots']))
            self.merge_error = False

        elif self.request.path.endswith('create/'):
            self.request.session["mass_data"] = mass_data
            self.request.session["k_data"] = k_data
            self.request.session['distances'] = [distances]
            self.request.session['input_data'] = [form.cleaned_data]
            self.request.session['min_M'] = form.cleaned_data['min_M']
            self.request.session['max_M'] = form.cleaned_data['max_M']
            self.request.session['M_step'] = form.cleaned_data['M_step']
            self.request.session['warnings'] = warnings
            self.request.session['extra_plots'] = form.cleaned_data['extra_plots']


        return super(HMFInputBase, self).form_valid(form)


class HMFInputParent(BaseTab):
    _is_tab = True
    tab_id = 'form-parent'
    tab_label = 'Calculate'
    template_name = 'also_doesnt_exist.html'
    my_children = ['/hmf_finder/form/create/', '/hmf_finder/form/add/']


class HMFInputChild(BaseTab):
    """Base class for all child navigation tabs."""
    tab_parent = HMFInputParent


class HMFInputCreate(HMFInputBase, HMFInputChild):

    #This defines whether to add the M-bounds fields to the form (here we do)
    def get_form_kwargs(self):
        kwargs = super(HMFInputBase, self).get_form_kwargs()
        kwargs.update({
            'add' : 'create'
        })
        return kwargs

    #We must define 'get' as TabView is based on TemplateView which has a different form for get (form variable lost).
    def get(self, request, *args, **kwargs):
        """
        Handles GET requests and instantiates a blank version of the form.
        """
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        return self.render_to_response(self.get_context_data(form=form))

    #TabView-specific things
    _is_tab = True
    tab_id = '/hmf_finder/form/create/'
    tab_label = 'Begin New'
    top = False



class HMFInputAdd(HMFInputBase, HMFInputChild):

    def get_form_kwargs(self):
        kwargs = super(HMFInputBase, self).get_form_kwargs()
        kwargs.update({
             'add' : 'add',
             'minm' : self.request.session['min_M'],
             'maxm' : self.request.session['max_M']
        })
        return kwargs

    def get(self, request, *args, **kwargs):
        """
        Handles GET requests and instantiates a blank version of the form.
        """
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        return self.render_to_response(self.get_context_data(form=form))

    _is_tab = True
    tab_id = '/hmf_finder/form/add/'
    top = False
    tab_label = "Add Plots"
    def tab_visible(self):
        return "extrapolate" in self.request.session


class ViewPlots(BaseTab):

    def collect_dist(self, distances):
        final_d = []
        collected_cosmos = []
        collected_z = []
        for dist in distances:  # dist is one matrix of distances
            for d in dist:  # d is one vector (different calculations - a row of final table)
                if d[0] in collected_cosmos and d[1] in collected_z:
                    b = [item for item in range(len(collected_cosmos)) if collected_cosmos[item] == d[0]]
                    if d[1] in b:
                        break
                    else:
                        collected_cosmos = collected_cosmos + [d[0]]
                        collected_z = collected_z + [d[1]]
                collected_cosmos = collected_cosmos + [d[0]]
                collected_z = collected_z + [d[1]]
                final_d = final_d + [d]
        return final_d

    def get(self, request, *args, **kwargs):
        self.form = forms.PlotChoice(request)
        distances = request.session['distances']
        self.warnings = request.session['warnings']
        self.final_dist = self.collect_dist(distances)
        return self.render_to_response(self.get_context_data(form=self.form, distances=self.final_dist, warnings=self.warnings))

    template_name = 'hmf_image_page.html'
    _is_tab = True
    tab_id = '/hmf_finder/hmf_image_page/'
    tab_label = 'View Plots'
    top = True

    def tab_visible(self):
        return "extrapolate" in self.request.session

def plots(request, filetype, plottype):
    """
    Chooses the type of plot needed and the filetype (pdf or png) and outputs it
    """
    #TODO: give user an option for ylim dynamically?
    # Definitions of plot-types
    mass_plots = ['hmf', 'f', 'ngtm', 'mhmf', 'comparison_hmf',
                  'comparison_f', 'Mgtm', 'nltm', 'Mltm', 'L',
                  'sigma', 'lnsigma', 'n_eff']

    k_plots = ['power_spec']
    print plottype
    # Get data from session object
    if plottype in mass_plots:
        mass_data = request.session["mass_data"]
        masses = mass_data["M"]
        xlab = r'Mass $(M_{\odot}h^{-1})$'
    elif plottype in k_plots:
        k_data = request.session["k_data"]

    # DEFINE THE FUNCTION TO BE PLOTTED
    if plottype in mass_plots:
        if plottype == 'hmf':
            keep = [string for string in mass_data.columns if string.startswith("hmf_")]
            mass_data = mass_data[keep]
            title = 'Mass Function'
            ylab = r'Logarithmic Mass Function $\log_{10} \left( \frac{dn}{d \ln M} \right) h^3 Mpc^{-3}$'
            yscale = 'log'

        elif plottype == 'f':
            keep = [string for string in mass_data.columns if string.startswith("f(sig)_")]
            # mass_data[i].keep_columns(keep)
            mass_data = mass_data[keep]
            title = 'Fraction of Mass Collapsed'
            ylab = r'Fraction of Mass Collapsed, $f(\sigma)$'
            yscale = 'linear'

        elif plottype == 'ngtm':
            keep = [string for string in mass_data.columns if string.startswith("NgtM_")]
            # mass_data.keep_columns(keep)
            mass_data = mass_data[keep]
            title = 'n(>M)'
            ylab = r'$\log_{10}(n(>M)) h^3 Mpc^{-3}$'
            yscale = 'linear'

        elif plottype == 'Mgtm':
            keep = [string for string in mass_data.columns if string.startswith("MgtM_")]
            # mass_data.keep_columns(keep)
            mass_data = mass_data[keep]
            title = 'Total Bound Mass in Haloes Greater Than M'
            ylab = r'Mass(>M), $\log_{10} M_{sun}h^{2}Mpc^{-3}$'
            yscale = 'linear'

        elif plottype == 'nltm':
            keep = [string for string in mass_data.columns if string.startswith("NltM_")]
            # mass_data.keep_columns(keep)
            mass_data = mass_data[keep]
            title = 'n(<M)'
            ylab = r'$\log_{10}(n(>M)) h^3 Mpc^{-3}$'
            yscale = 'linear'

        elif plottype == 'Mltm':
            keep = [string for string in mass_data.columns if string.startswith("MltM_")]
            # mass_data.keep_columns(keep)
            mass_data = mass_data[keep]
            title = 'Total Bound Mass in Haloes Smaller Than M'
            ylab = r'Mass(<M), $\log_{10} M_{sun}h^{2}Mpc^{-3}$'
            yscale = 'linear'

        elif plottype == 'mhmf':
            keep = [string for string in mass_data.columns if string.startswith("M*hmf_")]
            # mass_data.keep_columns(keep)
            mass_data = mass_data[keep]
            title = 'Mass by Mass Function'
            ylab = r'Mass by Mass Function $\left( M\frac{dn}{d \ln M} \right) M_{sun} h^3 Mpc^{-3}$'
            yscale = 'linear'

        elif plottype == 'comparison_hmf':
            keep = [string for string in mass_data.columns if string.startswith("hmf_")]
            # mass_data.keep_columns(keep)
            mass_data = mass_data[keep]
            mass_data = mass_data
            first_column = mass_data.columns[0]
            mass_data = mass_data.div(mass_data[first_column], axis=0)
            yscale = 'linear'
            title = 'Comparison of Mass Functions'
            ylab = r'Ratio of Logarithmic Mass Functions $ \left(\frac{dn}{d \ln M}\right) / \left( \frac{dn}{d \ln M} \right)_0 $'

        elif plottype == 'comparison_f':
            keep = [string for string in mass_data.columns if string.startswith("f(sig)_")]
            mass_data = mass_data[keep]

            first_column = mass_data.columns[0]
            mass_data = mass_data.div(mass_data[first_column], axis=0)
            mass_data = mass_data
            yscale = 'linear'
            title = 'Comparison of Fitting Function(s)'
            ylab = r'Fitting Function $ (f(\sigma)/ f(\sigma)_0)$'



        elif plottype == 'L':
            keep = [string for string in mass_data.columns if string.startswith("L(N=1)_")]
            # mass_data.keep_columns(keep)
            mass_data = mass_data[keep]
            title = "Box Size, L, required to expect one halo"
            ylab = "Box Size, L (Mpc/h)"
            yscale = 'log'

        elif plottype == 'sigma':
            keep = [string for string in mass_data.columns if string.startswith("sigma_")]
            # mass_data.keep_columns(keep)
            mass_data = mass_data[keep]
            title = "Mass Variance"
            ylab = r'Mass Variance, $\sigma$'
            yscale = 'linear'

        elif plottype == 'lnsigma':
            keep = [string for string in mass_data.columns if string.startswith("lnsigma_")]
            # mass_data.keep_columns(keep)
            mass_data = mass_data[keep]
            title = "Logarithm of Inverse Sigma"
            ylab = r'$\ln(\sigma^{-1})$'
            yscale = 'linear'

        elif plottype == 'n_eff':
            keep = [string for string in mass_data.columns if string.startswith("neff_")]
            # mass_data.keep_columns(keep)
            mass_data = mass_data[keep]
            title = "Effective Spectral Index"
            ylab = r'Effective Spectral Index, $n_{eff}$'
            yscale = 'linear'

        canvas = utils.create_canvas(masses, mass_data, title, xlab, ylab, yscale)

    else:
        xlab = "Wavenumber"
        if plottype == 'power_spec':
            k_keys = [string for string in k_data.columns if string.startswith("k_")]
            p_keys = [string for string in k_data.columns if string.startswith("P(k)_")]

            title = "Power Spectra"
            ylab = "Power"

        canvas = utils.create_k_canvas(k_data, k_keys, p_keys, title, xlab, ylab)


    # How to output the image
    if filetype == 'png':
        response = HttpResponse(content_type='image/png')
        canvas.print_png(response)
    elif filetype == 'pdf':
        response = HttpResponse(content_type='application/pdf')
        response["Content-Disposition"] = "attachment;filename=" + plottype + ".pdf"
        canvas.print_pdf(response)

    elif filetype == 'zip':
        response = StringIO.StringIO()
        canvas.print_pdf(response)

    return response

def header_txt(request):
    # Set up the response object as a text file
    response = HttpResponse(mimetype='text/plain')
    response['Content-Disposition'] = 'attachment; filename=parameters.dat'

    # Import all the input form data so it can be written to file
    formdata = request.session['input_data']

    # Write the parameter info
    response.write('# File Created On: ' + str(datetime.datetime.now()) + '\n')
    response.write('# With version ' + settings.version + ' of HMFcalc \n')
    response.write('# And version ' + version + ' of hmf (backend) \n')
    response.write('# \n')
    response.write('# SETS OF PARAMETERS USED \n')
    response.write('# The following blocks indicate sets of parameters that were used in all combinations' + '\n')
    for data in formdata:
        response.write('# =====================================================\n')
        response.write('# Redshifts: ' + str(data['z']) + '\n')
        response.write('# WDM Masses: ' + str(data['WDM']) + '\n')
        response.write('# Fitting functions: ' + str(data['approach']) + '\n')
        response.write('# Virial Overdensity: ' + str(data['overdensity']) + '\n')
        response.write('# Transfer Function: ' + str(data['co_transfer_file']) + '\n')
        if data['extrapolate']:
            response.write('# Minimum k: ' + str(data['k_begins_at']) + '\n')
            response.write('# Maximum k: ' + str(data['k_ends_at']) + '\n')

        response.write('# Cosmologies: \n')
        response.write('# \n')
        for j, cosmo in enumerate(data['cp_label']):
            response.write('# ' + cosmo + ': \n')
            response.write('# -----------------------------------------------------\n')
            response.write('# Critical Overdensity: ' + str(data['cp_delta_c'][min(j, len(data['cp_delta_c']) - 1)]) + '\n')
            response.write('# Power Spectral Index: ' + str(data['cp_n'][min(j, len(data['cp_n']) - 1)]) + '\n')
            response.write('# Sigma_8: ' + str(data['cp_sigma_8'][min(j, len(data['cp_sigma_8']) - 1)]) + '\n')
            response.write('# Hubble Parameter: ' + str(data['cp_H0'][min(j, len(data['cp_H0']) - 1)]) + '\n')
            response.write('# Omega_b: ' + str(data['cp_omegab'][min(j, len(data['cp_omegab']) - 1)]) + '\n')
            response.write('# Omega_CDM: ' + str(data['cp_omegac'][min(j, len(data['cp_omegac']) - 1)]) + '\n')
            response.write('# Omega_Lambda: ' + str(data['cp_omegav'][min(j, len(data['cp_omegav']) - 1)]) + '\n')
            response.write('# w: ' + str(data['cp_w_lam'][min(j, len(data['cp_w_lam']) - 1)]) + '\n')
            response.write('# Omega_nu: ' + str(data['cp_omegan'][min(j, len(data['cp_omegan']) - 1)]) + '\n')
            response.write('# -----------------------------------------------------\n')
        response.write('# =====================================================\n')
        response.write('# \n')

        return response

def hmf_txt(request):
    # Import all the data we need
    mass_data = request.session["mass_data"]

    # Set up the response object as a text file
    response = HttpResponse(mimetype='text/plain')
    response['Content-Disposition'] = 'attachment; filename=mass_functions.dat'

    table = mass_data.to_string(index_names=False, index=False)
    response.write(table)

    return response

def power_txt(request):
    # Import all the data we need
    k_data = request.session["k_data"]

    # Set up the response object as a text file
    response = HttpResponse(mimetype='text/plain')
    response['Content-Disposition'] = 'attachment; filename=power_spectra.dat'

    table = k_data.to_string(index_names=False, index=False)
    response.write(table)

    return response

def hmf_all_plots(request):


    # First make all the canvases...
    mass_func_file_like = plots(request, filetype='zip', plottype='hmf')
    f_file_like = plots(request, filetype='zip', plottype='f')
    ngtm_file_like = plots(request, filetype='zip', plottype='ngtm')
    mhmf_file_like = plots(request, filetype='zip', plottype='mhmf')
    comparison_mf_file_like = plots(request, filetype='zip', plottype='comparison_hmf')
    comparison_f_file_like = plots(request, filetype='zip', plottype='comparison_f')
    mgtm_file_like = plots(request, filetype='zip', plottype='Mgtm')
    sigma_file_like = plots(request, filetype='zip', plottype='sigma')
    lnsigma_file_like = plots(request, filetype='zip', plottype='lnsigma')
    n_eff_file_like = plots(request, filetype='zip', plottype='n_eff')
    power_spec_file_like = plots(request, filetype='zip', plottype='power_spec')

    # ZIP THEM UP
    response = HttpResponse(mimetype='application/zip')
    response['Content-Disposition'] = 'attachment; filename=all_plots.zip'

    buff = StringIO.StringIO()
    archive = zipfile.ZipFile(buff, 'w', zipfile.ZIP_DEFLATED)
    archive.writestr('mass_functions.pdf', mass_func_file_like.getvalue())
    archive.writestr('fitting_functions.pdf', f_file_like.getvalue())
    archive.writestr('n_gt_m.pdf', ngtm_file_like.getvalue())
    archive.writestr('mass_by_mass_functions.pdf', mhmf_file_like.getvalue())
    archive.writestr('mass_function_comparison.pdf', comparison_mf_file_like.getvalue())
    archive.writestr('fitting_function_comparison.pdf', comparison_f_file_like.getvalue())
    archive.writestr('mgtm_comparison.pdf', mgtm_file_like.getvalue())
    archive.writestr('mass_variance.pdf', sigma_file_like.getvalue())
    archive.writestr('log_one_on_sigma.pdf', lnsigma_file_like.getvalue())
    archive.writestr('effective_spectral_index.pdf', n_eff_file_like.getvalue())
    archive.writestr('power_spectrum.pdf', power_spec_file_like.getvalue())
    archive.close()
    buff.flush()
    ret_zip = buff.getvalue()
    buff.close()
    response.write(ret_zip)
    return response



'''
Created on May 20, 2013

@author: Steven
'''
from setuptools import setup, find_packages
import os

version = '1.0.2'
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="HMFcalc",
    version=version,
    packages=find_packages(),
    include_package_data=True,
    install_requires=[],
    requires=['numpy', 'pandas'],
    author="Steven Murray",
    author_email="steven.jeanette.m@gmail.com",
    description="A halo mass function calculator web-app in django",
    long_description=read('README'),
    keywords="halo mass function",
    url="https://github.com/steven-murray/HMFcalc"
)
